/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

/* $Id: gllin.c $
 *
 * Global Locate driver for FIC GTA01 (OpenMoko Neo1973).
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <termios.h>
#include <time.h>
#include <stdio.h>
#include <wait.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifndef TRUE
#define TRUE (1)
#endif

#ifndef FALSE
#define FALSE (0)
#endif

#ifndef PATH_MAX
#define PATH_MAX        1024
#endif

typedef struct glData
{
    int pid;
    int fd;
} glData;

static glData GL_DATA =
{
     0,         // pid
    -1          // fd
};

#define pipeName        "/dev/ttySAC1"

extern const char* gllin_stop();
static void gllin_poweron(int on);
extern void gllin_arg_set(int arg_max, int* pArgc, const char** pp_argv);
extern void trace_arg(int argc, char** argv);
extern void pm_controls_active(bool active);
extern void runtime_init();

extern int debug_all;

//------------------------------------------------------------------------------------
//
//              isRunning()
//
//------------------------------------------------------------------------------------

static int isRunning()
{
    // printf("isRunning pid %d   fd %d\n", GL_DATA.pid, GL_DATA.fd);

    // if (GL_DATA.pid && (-1 != GL_DATA.fd))
    if ((-1 != GL_DATA.fd))
    {
        return 1;
    }
    return 0;
}

//------------------------------------------------------------------------------------
//
//              gllin_conf_read()
//
//------------------------------------------------------------------------------------

#define MAX_ARGS        20
extern int nmea_log;
int daemonize;

static void gllin_conf_read(char* path, int* pArgc, char** argv)
{
    static char buf[4000];

    memset(buf, 0, sizeof(buf));

    int    f = open("/etc/gllin/gllin.conf", O_RDONLY);
    int    n = read(f, buf, sizeof(buf) - 1);
    char*  p = buf;
    char*  pEnd;
    int    argc = 0;
    int    argMax = 0;
    int    dummy;
    int    skip_next_arg = 0;
    daemonize = 0;

    buf[sizeof(buf)-1] = 0;
    if (n > 0) n = strlen(buf);

    if (argv)
    {
        argv[0] = 0;
    }

    if (pArgc)
    {
        argMax = *pArgc - 1;
    }
    else
    {
        pArgc = &dummy;
    }
    *pArgc = 0;

    close(f);

    if (n <= 0)
    {
        strcpy(path, "./gllin");
        *pArgc = 0;
#ifdef DEBUG_LV1			
        printf("gllin conf[0] %s\n", path);
#endif
        printf("%s/%d path %s\n", __FILE__, __LINE__, path);
        return;
    }

    //  Get the path.
    {
        char* s = path;
        strncpy(s, p, PATH_MAX);
        s[PATH_MAX-1] = 0;

        while ((*s != ' ') && *s)
        {
            ++s;
        }
        if (*s == ' ')
        {
            *s = 0;
            p += (s - path);
        }
    }

    //  Split the config string into arguments.

    if (argv) argv[argc++] = path;

    for (pEnd = p + strlen(p); (argc < argMax) && (p < pEnd); ++p )
    {
        char* arg;
        if (' ' == *p)
        {
            *p = 0;
            continue;
        }
        if (!*p)
        {
            continue;
        }

        if (argv) argv[argc] = p;
        do
        {
            ++p;
        } while ((*p != ' ') && (*p != '\n') && (*p != '\r') && *p);
        *p = 0;

        ++argc;
        arg = argv[argc-1];
        if (!nmea_log)
        {
            if (skip_next_arg)
            {
                argv[--argc] = 0;
                skip_next_arg = 0;
            }
            else if (strstr(arg, "log"))
            {
                argv[--argc] = 0;
                skip_next_arg = 1;
            }
        }
        if (!strcmp(arg, "+daemon"))
        {
            daemonize = 1;
        }
    }

    if (argv)
    {
        argv[argc] = 0;
    }
    *pArgc     = argc;
}

//------------------------------------------------------------------------------------
//
//              gllin_detect()
//
//   1)  Does the /etc/gllin/gllin.conf file exist?
//   2)  If so, parse the command line.  Does the executable exist?
//   3)  If not, parse the default command line.  Does THAT executable exist?
//
//------------------------------------------------------------------------------------

const char* gllin_detect(void)
{
    struct stat st;
    char   path[PATH_MAX];

    return NULL;

    if (isRunning())
    {
        return 0;
    }

    mkfifo(pipeName, O_CREAT | O_EXCL);
    chmod(pipeName, S_IRUSR | S_IWUSR |
                    S_IRGRP | S_IWGRP |
                    S_IROTH | S_IWOTH);

    gllin_conf_read(path, 0, 0);
 
    if (-1 == lstat(path, &st))
    {
        //int   err  = errno;                     /* UNUSED */
        //char* sErr = strerror(errno);           /* UNUSED */

        // printf("gllin: lstat(%s) Error %d %s\n", path, err, sErr);
        return "executable not found";
    }

    if (st.st_mode & S_IXOTH)
    {
#ifdef DEBUG_LV1			
        printf("gllin: gllin_detect OK %d\n",st.st_mode);
#endif
        return 0;
    }

#ifdef DEBUG_LV1			
    printf("gllin: gllin_detect FAIL %d\n",st.st_mode);
#endif
    return "gllin not executable";
}

//------------------------------------------------------------------------------
//
//              gllin_fork()
//
//      Fork and exec the GLLIN.
//
//------------------------------------------------------------------------------

static void
gllin_fork()
{
    int    argc = MAX_ARGS;
    char   path[PATH_MAX];
    char*  argv[MAX_ARGS+1];
    char** pargv = argv;
    int rc = 0;

    gllin_poweron(1);
    return;

    gllin_conf_read(path,     &argc, pargv);
    gllin_arg_set  (MAX_ARGS, &argc, const_cast<const char**> (argv));
    trace_arg(argc, argv);

    pm_controls_active(false);          // Turn off the main controls
    runtime_init();

    GL_DATA.pid = fork();
    if (!GL_DATA.pid)
    {
        close(GL_DATA.fd);
        GL_DATA.fd = -1;
        rc = execvp(path, argv);
        exit(2);
    }
    if (daemonize)
    {
        int stat;
        waitpid(GL_DATA.pid, &stat, 0);
    }
}

//------------------------------------------------------------------------------
//
//              gllin_start()
//
//------------------------------------------------------------------------------

#define DEFAULT_GLLIN_WD        "/tmp"

int gllin_fd(void)
{
    return GL_DATA.fd;
}

static int gllin_uart_speed(void)
{
	int ret;
	struct termios ti;

	ret = tcgetattr(GL_DATA.fd, &ti);
	if (ret<0)
		return ret;

	ret = cfsetispeed(&ti, B0);
	if (ret < 0)
		return ret;

	ret = cfsetospeed(&ti, B9600);
	if (ret < 0)
		return ret;

	ti.c_cflag &= ~CRTSCTS;
	ti.c_lflag &= ~ICANON;
	ti.c_lflag &= ~ECHO;

	return tcsetattr(GL_DATA.fd, TCSANOW, &ti);
}

const char* gllin_start(void)
{
    int running = isRunning();
    if (running)
    {
        return 0;
    }

    gllin_stop();

    GL_DATA.fd = open(pipeName, O_RDWR);
    if (gllin_uart_speed() < 0)
	    return "Fail to set speed";

#ifdef DEBUG_LV1			
    printf("gllin_start open(%s) error %s\n",
                                        pipeName, strerror(errno));
#endif
    if (-1 == GL_DATA.fd)
    {
        return "Can't open /tmp/nmeaNP";
    }

    const char* gllin_wd = getenv("GLLIN_WD");
    if (!gllin_wd) gllin_wd = DEFAULT_GLLIN_WD;
    if (chdir(gllin_wd))
    {
        return "Can't reach GLLIN_WD";
    }

    gllin_fork();

    printf("gllin_start pid %d fd %d\n", GL_DATA.pid, GL_DATA.fd);

    if (isRunning()) return 0;
    return "Failed to start gllin";
}

//------------------------------------------------------------------------------
//
//              gllin_stop()
//
//------------------------------------------------------------------------------

extern void page_main_stop(const char* msg);

const char* gllin_stop(void)
{
    page_main_stop(0);
    pm_controls_active(true);           // Turn ON the main controls
    close(GL_DATA.fd);
    GL_DATA.fd = -1;
    gllin_poweron(0);
    return 0;
}

//------------------------------------------------------------------------------
//
//              gta02
//
//------------------------------------------------------------------------------

static ssize_t bwrite(int fd, const void *buf, size_t count, int delay)
{
	size_t i;

	if (delay)
	{
		for (i = 0; i < count; i++)
		{
			if (write(fd, (unsigned char *) buf + i, 1) != 1)
				return -1;
			usleep(delay * 1000);
		}
	}
	else
		return write(fd, (unsigned char *) buf, count);

	return i;
}

static void gllin_ubx(char cls, char id, const unsigned char *ubx, int size, int delay)
{
	unsigned char header[6], checksum[2];
	int i;

	if (!GL_DATA.fd)
		return;

	header[0] = 0xb5;
	header[1] = 0x62;
	header[2] = cls;
	header[3] = id;
	header[4] = size & 0xff;
	header[5] = (size >> 8) & 0xff;

	checksum[0] = checksum[1] = 0;
	for (i = 2; i < 6; i++)
	{
		checksum[0] += header[i];
		checksum[1] += checksum[0];
	}

	for (i = 0; i < size; i++)
	{
		checksum[0] += ubx[i];
		checksum[1] += checksum[0];
	}

	if (bwrite(GL_DATA.fd, header, 6, delay) != 6)
		goto fail;

	if (size)
	{
		if (bwrite(GL_DATA.fd, (void *) ubx, size, delay) != size)
			goto fail;
	}

	if (bwrite(GL_DATA.fd, checksum, 2, delay) != 2)
		goto fail;

	if (debug_all)
	{
		printf("ubx sent: ");
		for (i = 0; i < 6; i++)
			printf("%02x ", header[i]);
		if (size)
		{
			for (i = 0; i < size; i++)
				printf("%02x ", ubx[i]);
		}

		printf("%02x %02x\n", checksum[0], checksum[1]);
	}

	return;

fail:
	printf("ubx failed\n");
}

#if 0
static void gllin_nmea(const char *nmea)
{
	char buf[1024];
	const char *p = nmea;
	int n, checksum = 0;

	if (!GL_DATA.fd)
		return;

	while (*p)
		checksum ^= *p++;

	n = snprintf(buf, 256, "$%s*%02x\r\n", nmea, checksum);

	if (bwrite(GL_DATA.fd, nmea, n, 0) != n)
		printf("nmea failed\n");

	printf("sent: %d: %s", n, buf);
}

static void gllin_poll_status(void)
{
	unsigned char buf[] = { 0x01, 0x03, 0x1 };

	gllin_ubx(0x06, 0x01, buf, sizeof(buf), 4);
}
#endif

static void gllin_poweron(int on)
{
	int fd;
	char cmd;

	fd = open("/sys/bus/platform/drivers/neo1973-pm-gps/neo1973-pm-gps.0/pwron", O_WRONLY);
	if (!fd)
		return;

	cmd = (on) ? '1' : '0';

	write(fd, &cmd, 1);

	close(fd);

	sleep(1);

	/* initialize */
	if (on)
	{
		//gllin_poll_status();
		runtime_init();
	}
}

void gllin_reset(char c)
{
	unsigned char ubx[4];
	int bbr;

	switch (c)
	{
	case 'c':
		bbr = 0x7ff;
		break;
	case 'w':
		bbr = 0x1;
		break;
	case 'h':
	default:
		bbr = 0x0;
		break;
	}

	ubx[0] = bbr & 0xff;
	ubx[1] = (bbr >> 8) & 0xff;
	ubx[2] = 0x8; /* stop GPS */
	ubx[3] = 0x0;

	gllin_ubx(0x06, 0x04, ubx, 4, 4);

	sleep(2);

	ubx[2] = 0x9; /* start GPS */
	gllin_ubx(0x06, 0x04, ubx, 4, 4);

	runtime_init();
}
