/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef LIB_LTO_H__
#define LIB_LTO_H__                     // {

#ifdef  __cplusplus
extern "C" {
#endif

//      liblto.h
//

//-----------------------------------------------------------------------------------
//
//      Apply Aiding to the GLL
//
//-----------------------------------------------------------------------------------

const char* ltoGetAiding(const char *lto_file_name);

//-----------------------------------------------------------------------------------
//
//      Given an LTO file, check the expiration:
//
// Parameters:
//      [input]  lto_file_name
//      [output] *hours        Filled with a positive number if LTO is good.
//                             Negative number if LTO is expired.
//      [output] userMessage   Pointer to formatted string that is filled with
//                             a description of LTO validity.
//      [input]  buf_size      Size of the user message buffer.
//
// Returns:
//      const char* errmsg:
//              If 0, then there was no error.
//              If non-zero, then the LTO is not valid or readable for
//              the reason described in the string.
//              For error return cases, the hours might be filled in
//              with the hours since expiration or the hours until the
//              LTO becomes valid.
//
//-----------------------------------------------------------------------------------

const char* ltoGetExpiration(const char *lto_file_name,
                             int*  hours,
                             char* userMessage,
                             unsigned long buf_size);

#ifdef  __cplusplus
}
#endif

#endif  // LIB_LTO_H__                     // }
