/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <fcntl.h>
#include <string.h>
#include <glib/gstdio.h>
#include <glib/gi18n.h>
#include "agps.h"
#include "config.h"

GtkWidget* notebook_create(GtkWidget* window);

extern GtkLabel* agpsLabel;
extern void NMEA_MSG(const char* msg);

void 
button_on(GtkWidget *button, gboolean sensitive)
{
    gtk_widget_set_sensitive(button, sensitive);
}

void
trace_arg(int argc, char** argv)
{
    FILE* f = fopen("omgui.trace", "a+");
    int   i = 0;

    fprintf(f, "\nCommand: ");
    for (; i < argc; ++i)
    {
        fprintf(f, "\"%s\" ", argv[i]);
        printf(      "%s ",   argv[i]);
    }
    fprintf(f, "\n");
     printf(   "\n");
    fclose(f);
}

void
trace_env(char** envp)
{
    FILE* f = fopen("omgui.trace", "a+");
    for (; *envp; ++envp)
    {
        fprintf(f, "env:  \"%s\"\n", *envp);
    }
    fclose(f);
}

const char* argv0;

FILE *dump_nmea_fp;
static int dump_nmea;
int debug_all;
static GOptionEntry entries[] = {
	{ "debug-nmea", 'd', 0, G_OPTION_ARG_NONE, &dump_nmea, NULL, NULL },
	{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &debug_all, NULL, NULL },
	{ NULL }
};

int main(int argc, char** argv, char** envp)
{
    GtkWidget* window;

    argv0 = argv[0];

    // printf("%s/%d\n", __FILE__, __LINE__);

    // trace_arg(argc, argv);
    // trace_env(envp);

    bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
    textdomain(GETTEXT_PACKAGE);

    if (!g_thread_supported())
    {
        g_thread_init(NULL);
    }

    gdk_threads_init();
    gtk_init(&argc, &argv);

    {
	    GOptionContext *context = g_option_context_new("gps");

	    g_option_context_add_main_entries(context, entries, NULL);
	    g_option_context_parse(context, &argc, &argv, NULL);
	    g_option_context_free(context);

	    if (debug_all)
		    dump_nmea_fp = stdout;

	    if (dump_nmea)
	    {
		    char name[128];
		    time_t t;
		    struct tm *tmp;
		    int fd;

		    if (g_chdir("/home/root") == 0)
		    {
			    strcpy(name, "nmea-");
			    t = time(NULL);
			    tmp = localtime(&t);

			    strftime(name + 5, sizeof(name) - 5, "%m%d-%H%M", tmp);

			    strcat(name, ".log");

			    fd = open(name, O_CREAT | O_WRONLY | O_SYNC);
			    if (fd >= 0)
				    dump_nmea_fp = fdopen(fd, "w");
		    }
	    }
    }
    //--------------------------------------------------------------------------
    //          top window
    //--------------------------------------------------------------------------
    window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_title(GTK_WINDOW (window), _("AGPS Test Program"));
    gtk_window_set_default_size(GTK_WINDOW(window),480,640);
    g_signal_connect(G_OBJECT(window), "delete_event", gtk_main_quit, NULL);

    gtk_container_add(GTK_CONTAINER(window), notebook_create(window));

    gtk_widget_show(window);

    gdk_threads_enter();	 
    gtk_main();
    gdk_threads_leave();

    return 0;
}
