/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>

#define PAGE_LTO
#define PAGE_MAP
#define PAGE_GSM
#define PAGE_STRESS
// #define PAGE_ADD
#define PAGE_RFI

#ifndef  PAGE_STRESS_H__                // {
#include "page_stress.h"
#endif                                  // }

extern const char* argv0;
GtkWidget* page_main_create (GtkWidget* window);
GtkWidget* page_SS_create   (GtkWidget* window);
GtkWidget* page_AZ_create   (GtkWidget* window);
GtkWidget* page_speed_create(GtkWidget* window);
GtkWidget* page_stats_create(GtkWidget* window);
GtkWidget* page_stress_create(GtkWidget* window);
GtkWidget* page_LTO_create(GtkWidget* window);
GtkWidget* page_MAP_create(GtkWidget* window);

#ifdef  PAGE_RFI
GtkWidget* page_bt_create(GtkWidget* window);
GtkWidget* page_audio_create(GtkWidget* window);
GtkWidget* page_rfi_create(GtkWidget* window);
#endif

#ifdef  PAGE_GSM
GtkWidget* page_GSM_create(GtkWidget* window);
#endif

#ifdef  PAGE_ADD
GtkWidget* page_ADD_create(GtkWidget* window);
#endif

GtkWidget* page_log_create(GtkWidget* window);

static GtkWidget* notebook = 0;

static void
on_notebook_3(GtkWidget *widget, gint arg1, gpointer data)
{
    // printf("%s/%d signal %s\n", __FILE__, __LINE__, (char*) data);
}

static void
on_notebook_4(GtkWidget *widget, gint arg1, guint page_num, gpointer data)
{
    // printf("%s/%d signal %s  page %d\n",
    //                     __FILE__, __LINE__,
    //                     (char*) data, page_num);
}

static void
notebook_callback(int cb_i, const char* signal)
{
    GCallback cb = G_CALLBACK(on_notebook_3);
    if (4 == cb_i) cb = G_CALLBACK(on_notebook_4);

    g_signal_connect(G_OBJECT(notebook), signal, cb, (gpointer) signal);
}

//      Bitmap for the pages to be created:

enum
{
        ALLOW_MAIN      = 0x01,            // part of minimum
        ALLOW_SS        = 0x02,            // part of minimum
        ALLOW_AZ        = 0x04,            // part of minimum
        ALLOW_SPEED     = 0x08,
        ALLOW_STATS     = 0x10,
        ALLOW_STRESS    = 0x20,
        ALLOW_LTO       = 0x40,        // part of minimum
        ALLOW_MAP       = 0x80,        // part of minimum
        ALLOW_ADD       = 0x100,

        ALLOW_MINIMUM   = (ALLOW_MAP+ALLOW_MAIN+ALLOW_SS+ALLOW_AZ)
};

static int notebook_page_mask = 0;

static const char*
page_names[] =
{
    "main",
    "signal strength",
    "azimuth",
    "dials",
    "test",
    "stats",
    "LTO",
    "plot"
};

#ifndef DIM
#define DIM(x)          (sizeof(x)/sizeof(*x))
#endif

static const char*
notebook_page_name(int i)
{
    // printf("%s/%d notebook_page_name(%d) %d  ",
    //             __FILE__, __LINE__, i, DIM(page_names));
    if (i >= 0 && i < (int) DIM(page_names))
    {
        // printf("%s\n", page_names[i]);
        return page_names[i];
    }
    // printf(",<null>\n");
    return 0;
}

GtkWidget*
notebook_create(GtkWidget* window)
{
    // printf("%s/%d\n", __FILE__, __LINE__);
    notebook = gtk_notebook_new();
    GtkWidget* label;
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);

    int page_mask = 0;
    int nPages = 0;

    if (argv0 && *argv0)
    {
        const char key[] = {"-pagemask-"};
        const char* p = strstr(argv0,key);
        if (p)
        {
            p += strlen(key);
            page_mask = strtol(p, 0, 16);
        }
    }
    page_mask |= ALLOW_MINIMUM;
    notebook_page_mask = page_mask;

    if (page_mask & ALLOW_MAIN)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("run");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                                 page_main_create(window), label);
    }

    if (page_mask & ALLOW_SS)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("SS");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                    page_SS_create(window), label);
    }

    if (page_mask & ALLOW_AZ)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("AZ");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                    page_AZ_create(window), label);
    }

    if (page_mask & ALLOW_SPEED)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("dial");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                    page_speed_create(window), label);
    }

    if (page_mask & ALLOW_STATS)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("stat");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook), 
                    page_stats_create(window), label);
    }

#ifdef PAGE_LTO
    if (page_mask & ALLOW_LTO)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("LTO");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                        page_LTO_create(window), label);
    }
#endif

#ifdef PAGE_MAP
    if (page_mask & ALLOW_MAP)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                                 page_MAP_create(window),
                                 gtk_label_new("plot")    );
    }
#endif

#ifdef PAGE_STRESS
    if (page_mask & ALLOW_STRESS)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("test");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                        page_stress_create(window), label);
    }
#endif

#ifdef PAGE_ADD
    if (page_mask & ALLOW_ADD)
    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                        page_ADD_create(window), gtk_label_new("ADD"));
    }
#endif

    {
        // printf("%s/%d\n", __FILE__, __LINE__);
        ++nPages;
        label = gtk_label_new("Log");
        gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
                    page_log_create(window), label);
    }

    if (nPages > 7)
    {
        gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);

        notebook_callback(3, "change-current-page");
        notebook_callback(3, "focus-tab");
        notebook_callback(3, "move-focus-out");
//      notebook_callback(4, "page-added");                // in GTK+ v2.10 and later
//      notebook_callback(4, "page-removed");              // in GTK+ v2.10 and later
//      notebook_callback(4, "page-reordered");            // in GTK+ v2.10 and later
//      notebook_callback(4, "reorder-tab");               // in GTK+ v2.10 and later
        notebook_callback(3, "select-page");
        notebook_callback(4, "switch-page");

    }
    gtk_widget_show(notebook);

    // printf("%s/%d\n", __FILE__, __LINE__);
    return notebook;
}

static bool b_exposedMN = false;        // Main page
static bool b_exposedSS = false;        // Signal strength
static bool b_exposedAZ = false;        // Azimuth
static bool b_exposedSP = false;        // Speed
static bool b_exposedST = false;        // Stats
static bool b_exposedRR = false;        // Stress
static bool b_exposedLT = false;        // LTO
static bool b_exposedMP = false;        // Map

bool exposedMN() { return b_exposedMN; }
bool exposedSS() { return b_exposedSS; }
bool exposedAZ() { return b_exposedAZ; }
bool exposedSP() { return b_exposedSP; }
bool exposedST() { return b_exposedST; }
bool exposedRR() { return b_exposedRR; }
bool exposedLT() { return b_exposedLT; }
bool exposedMP() { return b_exposedMP; }

extern void page_main_done();
extern void page_SS_done();
extern void page_AZ_done();
extern void page_speed_done();
extern void page_stats_done();
extern void page_stress_done();
extern void page_LTO_done();
extern void page_MAP_done();

void
notebook_nmea_epoch(void)
{
#if 0
    printf("notebook_nmea_epoch() %s%s%s%s%s\n",
                    (exposedMN()) ? "main" : "",
                    (exposedSS()) ? "SS" : "",
                    (exposedAZ()) ? "AZ" : "",
                    (exposedSP()) ? "speed" : "",
                    (exposedST()) ? "stats" : "",
                    (exposedST()) ? "LTO" : "",
                    (exposedMP()) ? "MAP" : "",
                    (exposedRR()) ? "stress" : "");
#endif
    if (exposedMN()) page_main_done();
    if (exposedSS()) page_SS_done();
    if (exposedAZ()) page_AZ_done();
    if (exposedSP()) page_speed_done();
    if (exposedST()) page_stats_done();
    if (exposedRR()) page_stress_done();
    if (exposedLT()) page_LTO_done();
    if (exposedMP()) page_MAP_done();
}

void
notebook_expose(const char* pageName)
{
    // printf("notebook_expose(%s)\n", pageName);
    b_exposedMN = false;
    b_exposedSS = false;
    b_exposedAZ = false;
    b_exposedSP = false;
    b_exposedST = false;
    b_exposedRR = false;
    b_exposedLT = false;
    if (!strcmp("MN", pageName)) b_exposedMN = true;
    if (!strcmp("SS", pageName)) b_exposedSS = true;
    if (!strcmp("AZ", pageName)) b_exposedAZ = true;
    if (!strcmp("SP", pageName)) b_exposedSP = true;
    if (!strcmp("ST", pageName)) b_exposedST = true;
    if (!strcmp("RR", pageName)) b_exposedRR = true;
    if (!strcmp("LT", pageName)) b_exposedLT = true;
    if (!strcmp("MP", pageName)) b_exposedMP = true;
}

void 
notebook_report_header(FILE* f, const char* msg)
{
    fprintf(f,
        "\n"
        "\n"
        "                          %s Page\n"
        "-------------------------------------------------------------------\n",
        msg);
}
#define HDR(m)          notebook_report_header(f,m)

const char* page_main_omgui_version(void);

void 
notebook_report_header(FILE* f)
{
    time_t t = time(0);
    fprintf(f,
        "\n"
        "     %s Report\n"
        "     Date:   %s\n"
        "\n"
        "                          Summary\n"
        "-------------------------------------------------------------------\n"
        "Pages enabled are %x:\n",

        page_main_omgui_version(),
        ctime(&t),
        notebook_page_mask);

    int page_count = 0;
    for (int bit = 0; bit < 32; ++bit)
    {
        const char* page_name = notebook_page_name(bit);
        if (page_name && ((1L << bit) & notebook_page_mask))
        {
            if (page_count > 3)
            {
                page_count = 0;
                fprintf(f, ",\n");
            }
            fprintf(f, "%s      %s",
                (page_count) ? "," : "", page_name);
            ++page_count;
        }
    }
    if (page_count) fprintf(f, "\n");
}

void 
notebook_report_trailer(FILE* f)
{
}

extern void page_main_report(FILE* f);
extern void page_SS_report(FILE* f);
extern void page_AZ_report(FILE* f);
extern void page_speed_report(FILE* f);
extern void page_stats_report(FILE* f);
extern void page_stress_report(FILE* f);
extern void page_LTO_report(FILE* f);
extern void page_map_report(FILE* f);
extern void page_RFI_report(FILE* f);
extern void page_audio_report(FILE* f);
extern void page_BT_report(FILE* f);
extern void page_GSM_report(FILE* f);
extern void page_ADD_report(FILE* f);

void
notebook_report(FILE* f)
{
    int page_mask = notebook_page_mask;
    notebook_report_header(f);

    if (page_mask & ALLOW_MAIN ) HDR("Main"           ), page_main_report (f);
    if (page_mask & ALLOW_SS   ) HDR("Signal Strength"), page_SS_report   (f);
    if (page_mask & ALLOW_AZ   ) HDR("Azimuth"        ), page_AZ_report   (f);
    // if (page_mask & ALLOW_SPEED) HDR("Dials"          ), page_speed_report(f);
    if (page_mask & ALLOW_STATS) HDR("Statistics"     ), page_stats_report(f);
#ifdef PAGE_LTO
    if (page_mask & ALLOW_LTO) HDR("LTO"), page_LTO_report(f);
#endif
#ifdef PAGE_MAP
    if (page_mask & ALLOW_MAP) HDR("Plot"), page_map_report(f);
#endif
#ifdef PAGE_STRESS
    if (page_mask & ALLOW_STRESS) HDR("Test"), page_stress_report(f);
#endif
#ifdef PAGE_GSM
    // if (page_mask & ALLOW_GSM) HDR("GSM"), page_GSM_report(f);
#endif
#ifdef PAGE_ADD
    // if (page_mask & ALLOW_ADD) HDR("ADD"), page_ADD_report(f);
#endif
#ifdef PAGE_RFI
    // if (page_mask & ALLOW_RFI  ) HDR("RFI"      ), page_RFI_report  (f);
    // if (page_mask & ALLOW_AUDIO) HDR("Audio"    ), page_audio_report(f);
    // if (page_mask & ALLOW_BT   ) HDR("BlueTooth"), page_BT_report   (f);
#endif
    notebook_report_trailer(f);
}
