/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#define MAX_LINES 40

enum {
	COLUMN_TIME,
	COLUMN_NMEA,
	N_COLUMNS
};

static GtkListStore *store;
static int num_lines;
static gboolean locked;

unsigned long int runtime_ms();

void page_log_append(const char *nmea)
{
	GtkTreeIter iter;

	if (!store || locked)
		return;

	if (num_lines >= MAX_LINES)
	{
		if (gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter))
		{
			if (gtk_list_store_remove(store, &iter))
				num_lines--;
		}
	}

	if (num_lines < MAX_LINES)
	{
		gtk_list_store_append(store, &iter);
		gtk_list_store_set(store, &iter, COLUMN_TIME, runtime_ms(), COLUMN_NMEA, nmea, -1);
		num_lines++;
	}
}

static GtkTreeModel *create_model(void)
{
	store = gtk_list_store_new(N_COLUMNS, G_TYPE_INT, G_TYPE_STRING);

	return GTK_TREE_MODEL(store);
}

#ifdef SELF_PRODUCING
static gboolean add_nmea(gpointer data)
{
	gchar buf[256];

	g_snprintf(buf, sizeof(buf), "%d", num_lines);

	page_log_append(buf);

	return TRUE;
}
#endif

static void on_lock_clicked(GtkWidget *w, gpointer data)
{
	locked = !locked;

	gtk_button_set_label(GTK_BUTTON(w), (locked) ? "Unlock" : "Lock");
}

GtkWidget*
page_log_create(GtkWidget* window)
{
	GtkWidget *vbox, *button, *scroll, *view;
	GtkTreeModel *model;
	GtkTreeViewColumn *column;
	GtkTreeSelection *selection;
	GtkCellRenderer *cell;

	model = create_model();
	view = gtk_tree_view_new_with_model(model);

	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(view));
	gtk_tree_selection_set_mode(selection, GTK_SELECTION_SINGLE);

	cell = gtk_cell_renderer_text_new();

	column = gtk_tree_view_column_new_with_attributes("Time", cell, "text", COLUMN_TIME, NULL);
	gtk_tree_view_column_set_min_width(GTK_TREE_VIEW_COLUMN(column), 80);
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

	column = gtk_tree_view_column_new_with_attributes("NMEA", cell, "text", COLUMN_NMEA, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(view), column);

	scroll = gtk_scrolled_window_new(NULL, NULL);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
	gtk_container_add(GTK_CONTAINER(scroll), view);

	button = gtk_button_new_with_label("Lock");
	g_signal_connect(button, "clicked", G_CALLBACK(on_lock_clicked), NULL);

	vbox = gtk_vbox_new(FALSE, 0);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), scroll);
	gtk_box_pack_start(GTK_BOX(vbox), button, FALSE, FALSE, 0);

	gtk_widget_show_all(vbox);

#ifdef SELF_PRODUCING
	g_timeout_add(500, add_nmea, NULL);
#endif

	return vbox;
}
