/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef  PAGE_STRESS_H__                // {
#include "page_stress.h"
#endif                                  // }

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#ifdef PAGE_STRESS

extern void create_range_controls(GtkWidget* window, GtkWidget* vbox, bool test);
extern const char* gllin_detect(void);
extern const char* gllin_start(void);
extern const char* gllin_stop(void);
extern void stress_cb_start(GtkWidget* label);
extern void clearStats();
extern void notebook_expose(const char* pageName);
extern bool doing_reset_gps;

extern void page_pm_batch_next();
extern void page_pm_fix_done();
extern void page_pm_batch_done();

GtkLabel* stressLabel;
extern void NMEA_MSG(const char* msg);

static gboolean
expose_stress_event(GtkWidget* w, GdkEventExpose* event)
{
    notebook_expose("RR");
    return FALSE;
}

static bool stress_test = false;
extern void page_main_gllin(bool start);

bool
page_stress_on(void)
{
    return stress_test;
}

void
page_stress_msg(const char* msg)
{
    gtk_label_set_text(stressLabel, msg);
}

void page_stress_done(void)
{
}

static void
on_clicked_reset_gps(GtkWidget* w, gpointer d)
{
    page_stress_msg("Resetting GPS ...");
    gllin_stop();
    doing_reset_gps = true;
    gllin_start();
    doing_reset_gps = false;
    sleep(1);
    gllin_stop();
    page_stress_msg("GPS device reset is completed.");
    page_pm_batch_done();
}

void
page_stress_msg_int(const char* msg, int n)
{
    char buf[88];
    sprintf(buf, "%s %d", msg, n);
    page_stress_msg(msg);
}

int
stress_rand(int min, int max)
{
    int result = min + rand() % (max-min);
    printf("stress_rand(%d,%d) %d\n", min, max, result);
    return result;
}

guint stress_timeout_id = 0;

// Caution:  called from a timer thread, so all called GTK+ widgets
//      must be protected by gdk_threads_enter() and gdk_threads_leave()

#if 0
static void
stress_stop()
{
    stress_timeout_id = 0;
    // gdk_threads_remove_timeout(stress_timeout_id);

    gllin_stop();
    page_stress_msg("Stopping");
}
#endif

//------------------------------------------------------------------------------------
//
//              Report
//
//  page_stress_do_report() - generate a report
//  page_stress_set_report_name() - update the report label on the button.
//  on_clicked_report() - start the generation of a report.
//
//------------------------------------------------------------------------------------

void notebook_report(FILE* f);

static int report_id = 0;
extern const char* page_main_test_name(void);

bool
page_stress_do_report(void)
{
    const char* test_name = page_main_test_name();
    if (!test_name) return false;

    char report_name[32];
    sprintf(report_name, "./log/%s-R-%d.txt", test_name, report_id);

    FILE* f = fopen(report_name, "a+");
    if (!f)
    {
        fprintf(stderr, "omgui: can't create report file %s:  %d %s\n",
                                        report_name, errno, strerror(errno));
        return false;
    }
    notebook_report(f);
    fclose(f);
    ++report_id;
    return true;
}

static GtkButton* wid_report = 0;

void
page_stress_set_report_name(void)
{
    if (wid_report)
    {
        char report_name[32];
        sprintf(report_name, "Report %s-R-%d", page_main_test_name(), report_id);
        gtk_button_set_label(wid_report, report_name);
    }
}

static void
on_clicked_report(GtkWidget* w, gpointer d)
{
    if (page_main_test_name())
    {
        page_stress_do_report();
        page_stress_set_report_name();
    }
}

// Caution:  MIGHT be called from a timer thread, so all called GTK+ widgets
//      must be protected by gdk_threads_enter() and gdk_threads_leave()

#ifdef OPT_STRESS_FEATURE                       // {

static void
stress_start(void)
{
    page_main_gllin(true);
    int timeout = stress_rand(2,600);      // run time is 2..600 seconds.
    stress_timeout_id = g_timeout_add_seconds(timeout, (gboolean (*)(void*))on_timeout_stress, 0);
    page_stress_msg_int("Started stress test", timeout);
}

static guint
on_timeout_stress(GtkWidget* w, gpointer d)
{
    if (!stress_timeout_id) return FALSE;
    stress_stop();

    int timeout = stress_rand(0,30);     // Sleep between 0 and 30 seconds;
    page_stress_msg_int("Stress delay", timeout);
    sleep(timeout);

    stress_start();
    return FALSE;
}

static void
on_clicked_stress(GtkWidget* w, gpointer d)
{
    stress_test = !stress_test;
    if (stress_test)
    {
        stress_start();
    }
    else
    {
        stress_stop();
    }
}
#endif

void
page_stress_RID()
{
    page_pm_batch_next();
}

void
page_stress_FIX()
{
    page_pm_fix_done();
}

void
page_stress_EXIT()
{
    // page_main_gllin(false);                  // don't have this yet.
    page_pm_batch_done();
    gllin_stop();
}

GtkWidget*
page_stress_create(GtkWidget* window)
{
    GtkWidget* label;
    GtkWidget* button;
    GtkWidget* table;

    //--------------------------------------------------------------------------
    //          table
    //--------------------------------------------------------------------------
    table = gtk_table_new(14, 2, TRUE);
    gtk_signal_connect(GTK_OBJECT(table), "expose_event",
                    G_CALLBACK(expose_stress_event), table);

    //--------------------------------------------------------------------------
    //          label
    //
    //  All text from the "stress" page goes to this stressLabel
    //--------------------------------------------------------------------------
    label = gtk_label_new("Future panel to show stress test results");
    stressLabel = GTK_LABEL(label);
    gtk_label_set_justify(stressLabel, GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 2, 1, 8);
    gtk_widget_show(label);

#ifdef OPT_STRESS_FEATURE
    //--------------------------------------------------------------------------
    //          start stress test button
    //--------------------------------------------------------------------------
    button = gtk_button_new_with_label("Start Stress Test");
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_clicked_stress), label);
    gtk_table_attach_defaults(GTK_TABLE (table), button, 0, 2, 8, 9);
    gtk_widget_show(button);
#endif

    //--------------------------------------------------------------------------
    //          range controls
    //--------------------------------------------------------------------------
    create_range_controls(window, table, true);

    //--------------------------------------------------------------------------
    //          reset GPS button
    //--------------------------------------------------------------------------
    button = gtk_button_new_with_label("Reset GPS");
    g_signal_connect(G_OBJECT(button), "clicked",
                     G_CALLBACK(on_clicked_reset_gps), label);
    gtk_table_attach_defaults(GTK_TABLE (table), button, 0, 1, 13, 14);
    gtk_widget_show(button);

    //--------------------------------------------------------------------------
    //          report button
    //--------------------------------------------------------------------------
    button = gtk_button_new_with_label("Report");
    g_signal_connect(G_OBJECT(button), "clicked",
                    G_CALLBACK(on_clicked_report), NULL);
    gtk_table_attach_defaults(GTK_TABLE (table), button, 1, 2, 13, 14);
    gtk_widget_show(button);
    wid_report = GTK_BUTTON(button);

    gtk_widget_show(table);
    return table;
}

#else                   // } {

void page_stress_msg(const char* msg)
{
}

void page_stress_done(void)
{
}

#endif                  // }


extern void pm_report(FILE* f, bool stress);

void
page_stress_report(FILE* f)
{
    pm_report(f, true);
}
