/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>

const char* start_type = "hot";
bool allow_lto = true;
extern bool doing_ptf;

bool doing_stress = false;
int  stress_count = -1;

static bool set_timeout = false;
static bool set_accuracy_mask = false;
static int  accuracy_mask = 50;
static int  timeout = 64;
static GtkWidget* wid_box_accuracy_mask;
static GtkWidget* wid_box_timeout;
static GtkAdjustment* wid_batch_adj;
static GtkAdjustment* wid_fix_adj;

//------------------------------------------------------------------------------
//
//      Range/Scale Widgets for OMGUI
//
//  Ken Yale
//  Global Locate
//
//  Ranges:
//      Fix Period 0...64, where 0 is native.
//      Fix Type:  HOT (default), WARM, COLD, SNR
//                 Fix type has option for LTO on or off.
//      Batch count 1..1000  [only if the "do batches" is active]
//      Fix   count 1..1000  [only if the "do batches" is active]
//
//  Special Cases:
//      If Batch count is 0, then do a OMGUI stress test (forever).
//      Start the GLLIN, then wait fix_count fixes (good or bad), then
//      stop the GLLIN, sleep 1 second & repeat forever.
//      In this case, if fix_count is 0, then the fix_count is computed
//      to be a random number between 10 and 265 (inclusive).
//
//  Buttons:
//      Do batches or not.
//      simulator used or not.
//      sensitivity/driving
//      logging on/off
//
//  Behavior:
//      fix period adjustment is shared between RUN and TEST pages.
//      Simulator button is shared between RUN and TEST pages.
//
//      All controls are disabled (but visible) when the GLLIN is running.
//
//      All BATCH_COUNT and FIX_COUNT are invisble until BATCH is on.
//
//------------------------------------------------------------------------------

static void cb_pos_menu_select( GtkWidget       *item,
                                GtkPositionType  pos )
{
    allow_lto  = true;
    start_type = (const char*) pos;

    if (strstr(start_type, "(no LTO)"))
    {
        static char tmp_start[32];
        strcpy(tmp_start, start_type);

        char* p = strchr(tmp_start, '(');
        p[-1] = 0;

        allow_lto = false;
        start_type = tmp_start;
    }

    printf("start type %s \"%s\"\n", start_type, (char*) pos);
}

int fix_rate = 0;

GtkAdjustment* wid_fix_rate0 = 0;
GtkAdjustment* wid_fix_rate1 = 0;

static void cb_fix_rate(GtkAdjustment* adj)
{
    static bool locked = false;

    if (!locked)
    {
        locked = true;
        if (adj != wid_fix_rate0) gtk_adjustment_set_value(wid_fix_rate0, adj->value);
        if (adj != wid_fix_rate1) gtk_adjustment_set_value(wid_fix_rate1, adj->value);
        locked = false;

        fix_rate = (gint) adj->value;
        printf("fix rate %d\n", fix_rate);
    }
}

static void
cb_accuracy_mask(GtkAdjustment* adj)
{
    set_accuracy_mask = true;
    accuracy_mask = (gint) adj->value;
    // printf("accuracy_mask %d\n", accuracy_mask);
}

static void
cb_timeout(GtkAdjustment* adj)
{
    set_timeout = true;
    timeout = (gint) adj->value;
    // printf("timeout %d\n", timeout);
}

int fix_count = 1;

static void cb_fix_count( GtkAdjustment* adj)
{
    fix_count = (gint) adj->value;
    // printf("fix count %d\n", fix_count);
}

int batch_count = 0;

static void cb_batch_count( GtkAdjustment* adj)
{
    batch_count = (gint) adj->value;
    // printf("batch count %d\n", batch_count);
}

GtkWidget* wid_batch_count;
GtkWidget* wid_fix_count;

bool doing_batch = false;
bool doing_reset_gps = false;

static void cb_batch_allowed(GtkToggleButton* button)
{
    printf("batch allowed %d\n", button->active);
    if (button->active)
    {
        gtk_widget_show(wid_batch_count);
        gtk_widget_show(wid_fix_count);
        doing_batch = true;
    }
    else
    {
        gtk_widget_hide(wid_batch_count);
        gtk_widget_hide(wid_fix_count);
        doing_batch = false;
    }
}

int nmea_log = TRUE;
GtkToggleButton* wid_log0;
GtkToggleButton* wid_log1;

static void cb_nmea_log_value( GtkToggleButton* button)
{
    nmea_log = button->active;
    printf("nmea_log %d\n", nmea_log);
    if (button != wid_log0) gtk_toggle_button_set_active(wid_log0, nmea_log);
    if (button != wid_log1) gtk_toggle_button_set_active(wid_log1, nmea_log);
}

int using_simulator = FALSE;

static void cb_simulator_value( GtkToggleButton* button)
{
    using_simulator = button->active;
    printf("simulator %d\n", using_simulator);
}

int pnd_mode = TRUE;
GtkToggleButton* wid_pnd0;
GtkToggleButton* wid_pnd1;

#if 0
static void cb_pnd_value( GtkToggleButton* button)
{
    pnd_mode = button->active;
    printf("pnd_mode %d\n", pnd_mode);
    if (button != wid_pnd0) gtk_toggle_button_set_active(wid_pnd0, pnd_mode);
    if (button != wid_pnd1) gtk_toggle_button_set_active(wid_pnd1, pnd_mode);
}
#endif

/* Convenience functions */

static GtkWidget* make_menu_item(const gchar* name, GCallback callback)
{
    GtkWidget* item = gtk_menu_item_new_with_label (name);
    g_signal_connect (G_OBJECT (item), "activate", callback, (gpointer)name);
    gtk_widget_show (item);

    return item;
}

void scale_init(GtkScale* scale)
{
    gtk_range_set_update_policy (GTK_RANGE(scale), GTK_UPDATE_CONTINUOUS);
    gtk_scale_set_digits     (scale, 0);
    gtk_scale_set_value_pos  (scale, GTK_POS_RIGHT);
    gtk_scale_set_draw_value (scale, TRUE);
}

static void cb_arrow_clicked(GtkButton* b, gpointer user_data)
{
    int offset = GPOINTER_TO_INT(user_data);
    gdouble value = gtk_adjustment_get_value(wid_fix_rate0);
    printf("arrow click offset %d + %f\n", offset, value);

    value += offset;
    gtk_adjustment_set_value(wid_fix_rate0, value);
    gtk_adjustment_set_value(wid_fix_rate1, value);
    // gtk_button_set_relief(b, GTK_RELIEF_NONE);
}

static GtkWidget* create_arrow_button(GtkArrowType a, int offset)
{
    GtkShadowType s = GTK_SHADOW_ETCHED_OUT;

    GtkWidget* button = gtk_button_new();
    GtkWidget* arrow  = gtk_arrow_new(a, s);
    gtk_container_add(GTK_CONTAINER(button), arrow);
    g_signal_connect (G_OBJECT (button), "clicked",
                      G_CALLBACK (cb_arrow_clicked), GINT_TO_POINTER(offset));
    gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
    gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);
    gtk_widget_show(arrow);
    gtk_widget_show(button);
    return button;
}

GtkWidget* wid_batch_mode = 0;
GtkWidget* wid_simulator  = 0;
GtkWidget* box_fix_type   = 0;
GtkWidget* wid_fix_box0   = 0;
GtkWidget* wid_fix_box1   = 0;

//      The gllin.cpp calls us

void
pm_controls_active(bool active)
{
    if (box_fix_type)  gtk_widget_set_sensitive(box_fix_type,      active);
    if (wid_simulator) gtk_widget_set_sensitive(wid_simulator,     active);
    if (wid_batch_mode)gtk_widget_set_sensitive(wid_batch_mode,    active);
    if (wid_batch_count) gtk_widget_set_sensitive(wid_batch_count, active);
    if (wid_fix_count) gtk_widget_set_sensitive(wid_fix_count,     active);
    if (wid_log0)      gtk_widget_set_sensitive(GTK_WIDGET(wid_log0), active);
    if (wid_log1)      gtk_widget_set_sensitive(GTK_WIDGET(wid_log1), active);
    if (wid_pnd0)      gtk_widget_set_sensitive(GTK_WIDGET(wid_pnd0), active);
    if (wid_pnd1)      gtk_widget_set_sensitive(GTK_WIDGET(wid_pnd1), active);
    if (wid_fix_box0)  gtk_widget_set_sensitive(wid_fix_box0,    active);
    if (wid_fix_box1)  gtk_widget_set_sensitive(wid_fix_box1,    active);
    if (wid_box_accuracy_mask)  gtk_widget_set_sensitive(wid_box_accuracy_mask,    active);
    if (wid_box_timeout)  gtk_widget_set_sensitive(wid_box_timeout,    active);
}

//-----------------------------------------------------------------------------------
//
//              Direct connection to the batch tests.
//
//      The NMEA is decoded and these functions are called when
//      a batch test starts, and when a fix is done.
//
//-----------------------------------------------------------------------------------

static int real_batch_count;
static int real_fix_count;

static void
update_batch_count()
{
    gtk_adjustment_set_value(wid_batch_adj, batch_count);
}

static void
update_fix_count()
{
    gtk_adjustment_set_value(wid_fix_adj, fix_count);
}

void
page_pm_batch_start()
{
    if (!doing_batch) return;
    real_batch_count = batch_count;
    real_fix_count   = fix_count;
    printf("page_pm_batch_start()/%d %d %d\n",
                __LINE__, real_batch_count, real_fix_count);
}

void
page_pm_batch_next()
{
    if (!doing_batch) return;
    fix_count = real_fix_count;
    --batch_count;
    printf("page_pm_batch_next()/%d %d %d\n",
                __LINE__, batch_count, fix_count);
    update_batch_count();
    update_fix_count();
}

void
page_pm_fix_done()
{
    if (!doing_batch) return;
    --fix_count;
    update_fix_count();
    // printf("page_pm_fix_done()/%d %d %d\n", __LINE__, batch_count, fix_count);
}

void
page_pm_batch_done()
{
    if (!doing_batch) return;
    batch_count = real_batch_count;
    fix_count   = real_fix_count;
    printf("page_pm_batch_done()/%d %d %d\n",
                __LINE__, batch_count, fix_count);
    update_fix_count();
    update_batch_count();
}

//-------------------------------------------------------------------------------------
//
//              create_range_controls()
//
//      Create the controls for the "run" and the "test" pages.
//      Some are shared.
//
//-------------------------------------------------------------------------------------

void create_range_controls(GtkWidget* window, GtkWidget* table, bool test)
{
    GtkWidget* box;
    GtkObject* adj;
    GtkWidget* scale;
    GtkWidget* label;

if (!test)
{
    //--------------------------------------------------------------------------
    //          Accuracy Mask
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    gtk_container_set_border_width (GTK_CONTAINER (box), 0);

    label = gtk_label_new("Accuracy Mask");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    adj = gtk_adjustment_new ((gdouble) accuracy_mask, 0.0, 1001.0, 1.0, 1.0, 1.0);
    g_signal_connect (G_OBJECT (adj), "value_changed",
                      G_CALLBACK (cb_accuracy_mask), (gpointer) adj);
    scale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
    scale_init(GTK_SCALE(scale));
    gtk_widget_show(scale);

    gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
    //gtk_widget_show(box);

    gtk_table_attach_defaults(GTK_TABLE (table), box, 0, 1, 11, 12);
    wid_box_accuracy_mask = box;

    //--------------------------------------------------------------------------
    //          Timeout (0..64)
    //          0 == forever.
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    gtk_container_set_border_width (GTK_CONTAINER (box), 3);

    label = gtk_label_new ("Timeout");
    gtk_box_pack_start(GTK_BOX (box), label, FALSE, FALSE, 0);
    gtk_widget_show(label);

    adj = gtk_adjustment_new((gdouble) timeout, 0.0, 65.0, 1.0, 1.0, 1.0);
    g_signal_connect (G_OBJECT (adj), "value_changed",
                      G_CALLBACK (cb_timeout), (gpointer) adj);
    scale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
    scale_init(GTK_SCALE(scale));
    gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
    gtk_widget_show(scale);

    gtk_table_attach_defaults(GTK_TABLE (table), box, 1, 2, 11, 12);
    //gtk_widget_show(box);

    wid_box_timeout = box;
}

    //--------------------------------------------------------------------------
    //          Logging checkbox (shared)
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    GtkWidget* button = gtk_check_button_new_with_label("Logging");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), nmea_log);
    g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (cb_nmea_log_value), NULL);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, TRUE, TRUE, 1);
    if (wid_log0) wid_log1 = GTK_TOGGLE_BUTTON(button);
    else          wid_log0 = GTK_TOGGLE_BUTTON(button);

    //gtk_widget_show(box);
    gtk_table_attach_defaults(GTK_TABLE (table), box, 1, 2, 12, 13);

    //--------------------------------------------------------------------------
    //          fix rate (0..64 seconds)
    //          0 == native.
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 0);
    gtk_container_set_border_width (GTK_CONTAINER (box), 0);

    label = gtk_label_new ("Fix rate");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 2);
    gtk_widget_show(label);

    button = create_arrow_button(GTK_ARROW_LEFT, -1);
    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);

    adj = gtk_adjustment_new ((gdouble) fix_rate, 0.0, 64.0, 1.0,
                                                            10.0, 10.0);
    g_signal_connect (G_OBJECT (adj), "value_changed",
                          G_CALLBACK (cb_fix_rate), (gpointer) adj);
    if (wid_fix_rate0) wid_fix_rate1 = GTK_ADJUSTMENT(adj);
    else               wid_fix_rate0 = GTK_ADJUSTMENT(adj);
    scale = gtk_hscale_new (GTK_ADJUSTMENT(adj));
    scale_init(GTK_SCALE(scale));
    gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
    gtk_widget_show(scale);

    button = create_arrow_button(GTK_ARROW_RIGHT, +1);
    gtk_box_pack_start(GTK_BOX(box), button, FALSE, FALSE, 0);

    if (wid_fix_box0) wid_fix_box1 = box;
    else              wid_fix_box0 = box;

    gtk_table_attach_defaults(GTK_TABLE (table), box, 0, 1, 12, 13);
    gtk_widget_show (box);

    if (!test) return;

    //--------------------------------------------------------------------------
    //          Fix start selection
    //          COLD, WARM, HOT, SNR Factory test.
    //          COLD+LTO, WARM+LTO
    //--------------------------------------------------------------------------

    box = gtk_hbox_new (FALSE, 6);
    box_fix_type = box;
    gtk_container_set_border_width (GTK_CONTAINER (box), 3);

    label = gtk_label_new ("Fix type");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    GtkWidget* opt  = gtk_option_menu_new();
    GtkWidget* menu = gtk_menu_new();

    GtkWidget* item = make_menu_item ("cold", G_CALLBACK (cb_pos_menu_select));
    gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);

    item = make_menu_item ("cold (no LTO)", G_CALLBACK (cb_pos_menu_select)); 
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

    item = make_menu_item ("warm", G_CALLBACK (cb_pos_menu_select)); 
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

    item = make_menu_item ("warm (no LTO)", G_CALLBACK (cb_pos_menu_select));
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

    item = make_menu_item ("hot", G_CALLBACK (cb_pos_menu_select));
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

    item = make_menu_item ("SNR", G_CALLBACK (cb_pos_menu_select));
    gtk_menu_shell_append (GTK_MENU_SHELL(menu), item);

    gtk_option_menu_set_menu (GTK_OPTION_MENU (opt), menu);
    gtk_option_menu_set_history(GTK_OPTION_MENU(opt), 4);    // [default] is "hot"
    gtk_box_pack_start (GTK_BOX (box), opt, TRUE, TRUE, 0);
    gtk_widget_show (opt);

    gtk_table_attach_defaults(GTK_TABLE (table), box, 1, 2, 11, 12);
    gtk_widget_show (box);

{
    //--------------------------------------------------------------------------
    //          Number of fixes (1..n)
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    gtk_container_set_border_width (GTK_CONTAINER (box), 0);

    label = gtk_label_new ("Fix count");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    adj = gtk_adjustment_new ((gdouble) fix_count, 0.0, 1100.0, 1.0, 1.0, 1.0);
    g_signal_connect (G_OBJECT (adj), "value_changed",
                      G_CALLBACK (cb_fix_count), (gpointer) adj);
    wid_fix_adj = GTK_ADJUSTMENT(adj);
    scale = gtk_hscale_new(wid_fix_adj);
    scale_init(GTK_SCALE(scale));
    gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
    gtk_widget_show (scale);

    gtk_table_attach_defaults(GTK_TABLE (table), box, 1, 2, 10, 11);
    wid_fix_count = box;

    //--------------------------------------------------------------------------
    //          Number of batches (0..n)
    //          0 == forever.
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    gtk_container_set_border_width (GTK_CONTAINER (box), 3);

    label = gtk_label_new ("Batches");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
    gtk_widget_show (label);

    adj = gtk_adjustment_new((gdouble) batch_count, 0.0, 1100.0, 1.0, 1.0, 1.0);
    g_signal_connect (G_OBJECT (adj), "value_changed",
                      G_CALLBACK (cb_batch_count), (gpointer) adj);
    wid_batch_adj = GTK_ADJUSTMENT(adj);
    scale = gtk_hscale_new(wid_batch_adj);
    scale_init(GTK_SCALE(scale));
    gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
    gtk_widget_show (scale);

    gtk_table_attach_defaults(GTK_TABLE (table), box, 0, 1, 10, 11);
    wid_batch_count = box;
}

    //--------------------------------------------------------------------------
    //          Batch mode checkbox
    //--------------------------------------------------------------------------
    box = gtk_hbox_new (FALSE, 6);
    button = gtk_check_button_new_with_label("Batch");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), FALSE);
    g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (cb_batch_allowed), NULL);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, TRUE, TRUE, 1);
    wid_batch_mode = button;

    //--------------------------------------------------------------------------
    //          Simulator checkbox
    //--------------------------------------------------------------------------
    button = gtk_check_button_new_with_label("Simulator");
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (button), using_simulator);
    g_signal_connect (G_OBJECT (button), "toggled",
                      G_CALLBACK (cb_simulator_value), NULL);
    gtk_widget_show(button);
    gtk_box_pack_start(GTK_BOX (box), button, TRUE, TRUE, 1);
    wid_simulator = button;

    gtk_widget_show(box);
    gtk_table_attach_defaults(GTK_TABLE (table), box, 0, 1, 11, 12);
}


//------------------------------------------------------------------------------
//
//      GLLIN parameter setting.
//
//------------------------------------------------------------------------------

static int argMax;
static int argc;
static const char** argv;

void addArg(const char* s)
{
    if (argc < argMax)
    {
        argv[  argc] = s;
        argv[++argc] = 0;
    }
}

void addArgInt(int n, char* s)
{
    if (argc < argMax)
    {
        sprintf(s, "%d", n);
        addArg(s);
    }
}

extern void on_clicked_start(GtkWidget* widget, gpointer data);

void
pm_controls_nmea_epoch_done(void)
{
    //      Trigger the stress test change...
    if (!doing_stress) return;

    if (--stress_count > 0) return;

    printf("pm_controls_nmea_epoch_done/%d %d %d\n",
                __LINE__, doing_stress, fix_count);

    on_clicked_start(0, 0);             // Stop
    sleep(2);                           // slight pause for GLLIN to exit.

    if (doing_ptf)
    {
        on_clicked_start(0, (gpointer) 1);  // Restart (single shot)
    }
    else
    {
        on_clicked_start(0, 0);             // Restart.
    }
}

void
gllin_arg_set(int arg_max, int* pArgc, const char** pp_argv)
{
    argc   = *pArgc;
    argMax = arg_max;
    argv   = pp_argv;

    if (doing_reset_gps)
    {
        addArg("-recover");
        *pArgc = argc;
        return;
    }

    if (doing_ptf)
    {
        addArg("+ptf");
        static char sTO[8];
        if (set_timeout)
        {
            addArg("-to");
            addArgInt(set_timeout, sTO);
        }

        static char sAM[8];
        if (set_accuracy_mask)
        {
            addArg("-am");
            addArgInt(accuracy_mask, sAM);
        }
    }
    else
    {
        static char sFixRate[4];
        addArg("-periodic");                    //  Periodic fix rate
        addArgInt(fix_rate, sFixRate);
    }

    doing_stress = false;

    if (doing_batch)
    {
        int count = fix_count;
        if (!count)
        {
            count = 10 + (rand() & 0xFF);
        }
        printf("%s/%d Starting %s test  count %d\n", __FILE__, __LINE__,
                        (batch_count) ? "batch" : "stress", count);
        if (batch_count)
        {
            static char sBatchCount[32];
            static char sFixCount[32];

            addArg("-batch");
            addArg(start_type);
            addArgInt(batch_count, sBatchCount);
            addArgInt(count,   sFixCount);
        }
        else
        {
            doing_stress = true;
            stress_count = count;
        }
    }
    else
    {
        addArg(start_type);
    }

    if (using_simulator) addArg("+SIM");
    if (!allow_lto)      addArg("-lto");
    if (nmea_log)        addArg("+nmea"); else addArg("-nmea");

    addArg("+np");

    *pArgc = argc;
}

void
pm_report(FILE* f, bool stress)
{
    if (stress)
    {
        if (doing_reset_gps)
        {
            fprintf(f, "Reset GPS hardware\n");
        }

        if (doing_stress)
        {
            fprintf(f, "Running stress test.  %d fixes%s\n",
                            stress_count,
                            (fix_count) ? "" : " (random)");
        }
        if (doing_batch)
        {
            fprintf(f, "Running batch test %d x %d (%d x %d remain)\n",
                            real_batch_count, real_fix_count,
                                 batch_count,      fix_count );
        }
        else
        {
            fprintf(f, "Not a batch test\n");
        }
    }
    else
    {
        if (doing_ptf)
        {
            fprintf(f, "Push to fix.\n");
            if (set_timeout)
                fprintf(f, "Position fix timeout %d seconds\n", set_timeout);
            if (set_accuracy_mask)
                fprintf(f, "Accuracy mask %d meters\n", accuracy_mask);
        }
        else if (fix_rate)
        {
            if (1 == fix_rate)
                fprintf(f, "Compute a fix every second\n");
            else 
                fprintf(f, "Compute a fix every %d seconds\n", fix_rate);
        }
        else
        {
            fprintf(f, "Compute a fix as fast as possible\n");
        }
        if (using_simulator)
            fprintf(f, "Connected to a GPS simulator.\n");
        if (!allow_lto)
            fprintf(f, "LTO is suppressed\n");
        if (!nmea_log)
            fprintf(f, "NMEA logging is suppressed\n");
    }

    if (strcmp(start_type,"hot"))
        fprintf(f, "Simulate a %s start\n", start_type);
    else
        fprintf(f, "Performing a %s start\n", start_type);
}
