/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef SV_BAR_H__
#define SV_BAR_H__

enum
{
    MIN_ELEVATION  = 10,    // The smallest legible angle.
    FONT_WIDTH  = 12,
    FONT_HEIGHT = 20,
    DB_HZ_to_DBM = -173,
    N_AVG = 60,
    MAX_SVS = 14,
    SEP_WIDTH = 3,          // 3 pixels between bars.
    BAR_HEIGHT = 300,       // 300 pixels tall.
    PIXMAP_WIDTH = 476,     // later --- get from the window somwhow.
    LARGE_WINDOW_WIDTH = PIXMAP_WIDTH - 80,
                            // slightly smaller than LARGE_WINDOW_HEIGHT??
    DEGREE_64 = 64         // Minimum arc drawing is 1/64 of a degree

};

typedef struct SV_bar
{
    int  prn;
    int  ss;
    int  ss_max;
    int  ss_min;
    int  az;
    int  el;
    bool use;
    int  avg[N_AVG];
    int  nAvg;
    int  sumAvg;
    int  indexAvg;
    unsigned long lastUpdate;
    unsigned long ss_max_time;
    unsigned long ss_min_time;

    void init()
    {
        nAvg     = 0;
        sumAvg   = 0;
        indexAvg = 0;
        use      = false;
        ss_max   =  0;
        ss_max_time = 0;
        ss_min   = 60;
        ss_min_time = 0;
    };

    void add_ss()
    {
        if (++indexAvg >= N_AVG)
        {
            indexAvg = 0;
        }
        if (++nAvg > N_AVG)
        {
            sumAvg -= avg[indexAvg];
        }
        avg[indexAvg] = ss;
        sumAvg += ss;
    };

    int avg_ss()
    {
        if (!use) return 0;
        if (nAvg < 1) return 0;
        if (nAvg < N_AVG) return sumAvg/nAvg;
        return (sumAvg - 1 + N_AVG/2) / N_AVG;
    };

    //  Return a number between -100 and +100 along the X and Y axes.
    void get_xy(int&x, int& y);
    static int scale_radius(int r);

    static int  nSS;
    static bool changed;
    static int  leastSamplesAveraged;
};

extern SV_bar bars[MAX_SVS];

#endif
